------------------------------------------------------------------------
-- DAC SPI interface. Ugly and hardcoded but easy on the host system  --
-- and fully standalone, no programming required. Made for a specific --
-- chip, TI TLV5617A that is installed on CENTIPEDE-BRK board and     --
-- hooked up to LSS11 on SPI1 interface.                              --
------------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tlv5617a_dac is
port (
	spi_ok		: in std_logic;
	dac_a_sent	: out std_logic;
	dac_b_sent	: out std_logic;
	dac_a_data	: in std_logic_vector(9 downto 0);
	dac_b_data	: in std_logic_vector(9 downto 0);
	ss_pin		: out std_logic;
	sclk_pin	: out std_logic;
	mosi_pin	: out std_logic;
	clk		: in std_logic;
	nrst		: in std_logic
);
end tlv5617a_dac;

architecture guts of tlv5617a_dac is

	-------------
	-- Signals --
	-------------

	type fsm_state is (IDLE, START, LOAD, LATCH, SHIFT);
	signal fsm_cur		: fsm_state;
	signal fsm_nxt		: fsm_state;
	
	signal thr		: std_logic_vector(15 downto 0);
	signal bit_cntr		: unsigned(3 downto 0);
	signal ab		: std_logic;	-- 0 - a, 1 - b
	signal do_ld_thr	: std_logic;
	signal do_shift		: std_logic;

begin

	--------------------------------------
	-- Top half of the FSM (sequential) --
	--------------------------------------
	fsm_top : process (nrst, clk, ab, do_ld_thr, do_shift, fsm_nxt)
	begin
		if (nrst='0') then
			thr <= (others => '0');
			bit_cntr <= (others => '0');
			ab <= '1';	-- Start with DAC_B
			fsm_cur <= IDLE;
		elsif (rising_edge(clk)) then
			if (do_ld_thr='1') then
				if (ab='0') then
					thr <= "1100" & dac_a_data & "00";
				else
					thr <= "0100" & dac_b_data & "00";
				end if;
				bit_cntr <= (others => '0');
				ab <= not ab;
			elsif (do_shift='1') then
				thr <= thr(14 downto 0) & '0';
				bit_cntr <= bit_cntr + 1;
			end if;
			fsm_cur <= fsm_nxt;
		end if;
	end process;
	
	--------------------------------------------
	-- Bottom half of the FSM (combinatorial) --
	--------------------------------------------
	fsm_bh : process (fsm_cur, bit_cntr, ab, spi_ok)
	begin
		do_ld_thr <= '0';
		do_shift <= '0';
		dac_a_sent <= '0';
		dac_b_sent <= '0';
		case fsm_cur is
			when IDLE =>
				fsm_nxt <= START;
			when START =>
				do_ld_thr <= '1';
				fsm_nxt <= LOAD;
			when LOAD =>
				fsm_nxt <= LATCH;
			when LATCH =>
				if (bit_cntr="1111") then
					if (spi_ok='1') then
						if (ab='0') then
							dac_b_sent <= '1';
						else
							dac_a_sent <= '1';
						end if;
					end if;
					fsm_nxt <= IDLE;
				else
					do_shift <= '1';
					fsm_nxt <= SHIFT;
				end if;
			when SHIFT =>
				fsm_nxt <= LATCH;
		end case;
	end process;

	ss_pin <= '1' when (fsm_cur=IDLE) else '0';
	sclk_pin <= '0' when (fsm_cur=LATCH) else '1';
	mosi_pin <= thr(15);
end guts;
